
#include "f2c.h"

static int c__9 = 9;
static int c__1 = 1;
static int c__3 = 3;
static int c__4 = 4;
static int c__2 = 2;

int reados_(
            char *flist, 
            int *iform, 
            int *n, 
            int *nmax, 
            real *eta, 
            real *u, 
            real *s, 
            int *nrp, 
            int *nwp, 
	          int *nhp, 
            real *rtl, 
            real *wsl, 
            real *hh, 
            real *ar, 
            real *ai, 
            int *nrx, 
            int *nwx, 
            int *nhx, 
            ftnlen flist_len)
{
  static char fmt_5000[] = "(a)";

  /* System generated locals */
  int eta_dim1, eta_offset, u_dim1, u_offset, s_dim1, s_offset, ar_dim1,
  ar_dim2, ar_offset, ai_dim1, ai_dim2, ai_offset, rtl_dim1, 
  rtl_offset, wsl_dim1, wsl_offset, i__1, i__2;
  olist o__1;
  cllist cl__1;

  int ih;
  char fname[80];
    
  static cilist io___1 = { 0, 6, 0, 0, 0 };
  static cilist io___3 = { 0, 10, 1, fmt_5000, 0 };

/* ---------------------------------------------------------------- */
/*     Reads Orr-Sommerfeld data files in binary or ascii format. */
/*     Data is spatial amplification complex wavenumber */
/*           ar(Re,w,H)  ai(Re,w,H) */
/*     stored on a R,W,H grid */
/*           R = ln(Re) */
/*           W = ln(w) - 0.5 ln(Re) */
/*           H = H */

/*     Input */
/*       FLIST   name of text file containing file prefixes to be read */
/*       IFORM   -1=unknown */
/*                0=binary */
/*                1=ascii */

/*     Output */
/*       N(h)      number of points across BL,  i=1..N */
/*       NMAX      max dimension of N */
/*       ETA(i,h)  BL y coordinate */
/*       U(i,h)    velocity profile */
/*       S(i,h)    shear profile dU/deta */
/*       NRP(h)    number of RTL values, r=1..NRP */
/*       NWP(h)    number of WSL values, w=1..NWP */
/*       NHP       number of H   values, h=1..NHP */
/*       RTL(r,h)  R  values */
/*       WSL(w,h)  W  values */
/*       HH(h)     H  values */
/*       AR(r,w,h) real wavenumber */
/*       AI(r,w,h) imaginary wavenumber */

/* ---------------------------------------------------------------- */
    
  /* Parameter adjustments */
  ai_dim1 = *nrx;
  ai_dim2 = *nwx;
  ai_offset = 1 + ai_dim1 * (1 + ai_dim2);
  ai -= ai_offset;
  ar_dim1 = *nrx;
  ar_dim2 = *nwx;
  ar_offset = 1 + ar_dim1 * (1 + ar_dim2);
  ar -= ar_offset;
  --hh;
  wsl_dim1 = *nwx;
  wsl_offset = 1 + wsl_dim1;
  wsl -= wsl_offset;
  rtl_dim1 = *nrx;
  rtl_offset = 1 + rtl_dim1;
  rtl -= rtl_offset;
  --nwp;
  --nrp;
  s_dim1 = *nmax;
  s_offset = 1 + s_dim1;
  s -= s_offset;
  u_dim1 = *nmax;
  u_offset = 1 + u_dim1;
  u -= u_offset;
  eta_dim1 = *nmax;
  eta_offset = 1 + eta_dim1;
  eta -= eta_offset;
  --n;

  /* Function Body */
  o__1.oerr = 0;
  o__1.ounit = 10;
  o__1.ofnmlen = flist_len;
  o__1.ofnm = flist;
  o__1.orl = 0;
  o__1.osta = "OLD";
  o__1.oacc = 0;
  o__1.ofm = 0;
  o__1.oblnk = 0;
  f_open(&o__1);

  s_wsle(&io___1);
  do_lio(&c__9, &c__1, "Reading...", (ftnlen)10);
  e_wsle();
    
  i__1 = *nhx;
  for (ih = 1; ih <= i__1; ++ih) 
  {
L5:
    i__2 = s_rsfe(&io___3);
    if (i__2 != 0) {
      goto L1001;
  }
  i__2 = do_fio(&c__1, fname, (ftnlen)80);
  if (i__2 != 0) {
    goto L1001;
  }
  i__2 = e_rsfe();
  if (i__2 != 0) {
    goto L1001;
  }

  /* ------ skip comment line */
	if (i_indx("#!", fname, (ftnlen)2, (ftnlen)1) != 0) 
  {
	    goto L5;
	}

/* ------ strip off leading blanks */
  // This doesn't look right.  Where is fname getting incremented? -rs
  
L10:
    
    if (*(unsigned char *)fname == ' ') 
    {
      s_copy(fname, fname + 1, (ftnlen)80, (ftnlen)79);
      goto L10;
    }
	
  reados1_(fname, iform, &n[ih], nmax, &eta[ih * eta_dim1 + 1], &u[ih * 
		u_dim1 + 1], &s[ih * s_dim1 + 1], &nrp[ih], &nwp[ih], &rtl[ih 
		* rtl_dim1 + 1], &wsl[ih * wsl_dim1 + 1], &hh[1], &ar[(ih * 
		ar_dim2 + 1) * ar_dim1 + 1], &ai[(ih * ai_dim2 + 1) * ai_dim1 
		+ 1], nrx, nwx, (ftnlen)80);

    }
    
    ih = *nhx + 1;

L1001:
    *nhp = ih - 1;
    
    cl__1.cerr = 0;
    cl__1.cunit = 10;
    cl__1.csta = 0;
    f_clos(&cl__1);

    return 0;
} /* reados_ */


int writos_(
            char *flist, 
            int *iform, 
            int *n, 
            int *nmax, 
            real *eta, 
            real *u, 
            real *s, 
            int *nrp, 
            int *nwp, 
            int *nhp, 
            real *rtl, 
            real *wsl, 
            real *hh, 
            real *ar, 
            real *ai, 
            int *nrx, 
            int *nwx, 
            int *nhx, 
            ftnlen flist_len)
{
    static char fmt_5000[] = "(a)";

    int eta_dim1, eta_offset, u_dim1, u_offset, s_dim1, s_offset, ar_dim1,
	     ar_dim2, ar_offset, ai_dim1, ai_dim2, ai_offset, rtl_dim1, 
	    rtl_offset, wsl_dim1, wsl_offset, i__1, i__2;
    olist o__1;
    cllist cl__1;

    /* Local variables */
    int ih;
    char fname[80];
    
    /* Fortran I/O blocks */
    static cilist io___5 = { 0, 6, 0, 0, 0 };
    static cilist io___7 = { 0, 10, 1, fmt_5000, 0 };


/* ---------------------------------------------------------------- */
/*     Writes Orr-Sommerfeld data files in binary or ascii format. */
/*     Data is spatial amplification complex wavenumber */
/*           ar(Re,w,H)  ai(Re,w,H) */
/*     stored on a R,W,H grid */
/*           R = ln(Re) */
/*           W = ln(w) - 0.5 ln(Re) */
/*           H = H */

/*     Input */
/*       FLIST   name of text file containing file prefixes to be read */
/*       IFORM   0=binary, ascii otherwise */
/*       N(h)      number of points across BL,  i=1..N */
/*       NMAX      max dimension of N */
/*       ETA(i,h)  BL y coordinate */
/*       U(i,h)    velocity profile */
/*       S(i,h)    shear profile dU/deta */
/*       NRP(h)    number of RTL values, r=1..NRP */
/*       NWP(h)    number of WSL values, w=1..NWP */
/*       NHP       number of H   values, h=1..NHP */
/*       RTL(r,h)  R  values */
/*       WSL(w,h)  W  values */
/*       HH(h)     H  values */
/*       AR(r,w,h) real wavenumber */
/*       AI(r,w,h) imaginary wavenumber */

/*     Output */
/*       written files */

/* ---------------------------------------------------------------- */

    /* Parameter adjustments */
    ai_dim1 = *nrx;
    ai_dim2 = *nwx;
    ai_offset = 1 + ai_dim1 * (1 + ai_dim2);
    ai -= ai_offset;
    ar_dim1 = *nrx;
    ar_dim2 = *nwx;
    ar_offset = 1 + ar_dim1 * (1 + ar_dim2);
    ar -= ar_offset;
    --hh;
    wsl_dim1 = *nwx;
    wsl_offset = 1 + wsl_dim1;
    wsl -= wsl_offset;
    rtl_dim1 = *nrx;
    rtl_offset = 1 + rtl_dim1;
    rtl -= rtl_offset;
    --nwp;
    --nrp;
    s_dim1 = *nmax;
    s_offset = 1 + s_dim1;
    s -= s_offset;
    u_dim1 = *nmax;
    u_offset = 1 + u_dim1;
    u -= u_offset;
    eta_dim1 = *nmax;
    eta_offset = 1 + eta_dim1;
    eta -= eta_offset;
    --n;

    o__1.oerr = 0;
    o__1.ounit = 10;
    o__1.ofnmlen = flist_len;
    o__1.ofnm = flist;
    o__1.orl = 0;
    o__1.osta = "OLD";
    o__1.oacc = 0;
    o__1.ofm = 0;
    o__1.oblnk = 0;
    f_open(&o__1);

    s_wsle(&io___5);
    do_lio(&c__9, &c__1, "Writing...", (ftnlen)10);
    e_wsle();
    
    i__1 = *nhx;
    for (ih = 1; ih <= i__1; ++ih) {

L5:
	i__2 = s_rsfe(&io___7);
	if (i__2 != 0) {
	    goto L1001;
	}
	i__2 = do_fio(&c__1, fname, (ftnlen)80);
	if (i__2 != 0) {
	    goto L1001;
	}
	i__2 = e_rsfe();
	if (i__2 != 0) {
	    goto L1001;
	}

  /* ------ skip comment line */
	if (i_indx("#!", fname, (ftnlen)2, (ftnlen)1) != 0) {
	    goto L5;
	}

/* ------ strip off leading blanks */
L10:
	if (*(unsigned char *)fname == ' ') 
  {
	    s_copy(fname, fname + 1, (ftnlen)80, (ftnlen)79);
	    goto L10;
	}

	writos1_(fname, iform, &n[ih], nmax, &eta[ih * eta_dim1 + 1], &u[ih * 
		u_dim1 + 1], &s[ih * s_dim1 + 1], &nrp[ih], &nwp[ih], &rtl[ih 
		* rtl_dim1 + 1], &wsl[ih * wsl_dim1 + 1], &hh[1], &ar[(ih * 
		ar_dim2 + 1) * ar_dim1 + 1], &ai[(ih * ai_dim2 + 1) * ai_dim1 
		+ 1], nrx, nwx, (ftnlen)80);
    }
    
    ih = *nhx + 1;

L1001:
    *nhp = ih - 1;
    
    cl__1.cerr = 0;
    cl__1.cunit = 10;
    cl__1.csta = 0;
    f_clos(&cl__1);
    cl__1.cerr = 0;
    cl__1.cunit = 9;
    cl__1.csta = 0;
    f_clos(&cl__1);

    return 0;
} /* writos_ */

int reados1_(
             char *fname, 
             int *iform, 
             int *n, 
             int *nmax, 
             real *eta, 
             real *u, 
             real *s, 
             int *nrp, 
             int * nwp, 
             real *rtl, 
             real *wsl, 
             real *hh, 
             real *ar, 
             real *ai, 
             int *nrx,
             int *nwx, 
             ftnlen fname_len)
{
  /* Format strings */
  static char fmt_2050[] = 
      "(\002 n =\002,i4,\002   H =\002,f7.3,\002   Ye"
      " =\002,f7.3,\002   dYi+1/dYi =\002,f6.3/)";

  /* System generated locals */
  int ar_dim1, ar_offset, ai_dim1, ai_offset, i__1, i__2, i__3;
  olist o__1;
  cllist cl__1;

  /* Local variables */
  static int i__, k, ir, iw;
  static real geo;
  static int kform;
  static real htest, rtemp, wtemp;
  static int ntest, irback, iwback;
  static real aitemp, artemp;
  static int iwlast;

  /* Fortran I/O blocks */
  static cilist io___10 = { 1, 9, 0, 0, 0 };
  static cilist io___14 = { 0, 6, 0, 0, 0 };
  static cilist io___15 = { 1, 9, 0, 0, 0 };
  static cilist io___16 = { 0, 9, 0, 0, 0 };
  static cilist io___18 = { 0, 9, 0, 0, 0 };
  static cilist io___19 = { 0, 9, 0, 0, 0 };
  static cilist io___20 = { 0, 9, 0, 0, 0 };
  static cilist io___21 = { 0, 9, 0, 0, 0 };
  static cilist io___23 = { 0, 9, 0, 0, 0 };
  static cilist io___25 = { 0, 9, 1, 0, 0 };
  static cilist io___26 = { 0, 9, 1, 0, 0 };
  static cilist io___28 = { 0, 6, 0, 0, 0 };
  static cilist io___29 = { 0, 6, 0, 0, 0 };
  static cilist io___30 = { 0, 9, 0, 0, 0 };
  static cilist io___31 = { 0, 9, 0, 0, 0 };
  static cilist io___32 = { 0, 9, 0, 0, 0 };
  static cilist io___33 = { 0, 9, 0, 0, 0 };
  static cilist io___34 = { 0, 9, 0, 0, 0 };
  static cilist io___35 = { 0, 9, 0, 0, 0 };
  static cilist io___36 = { 0, 9, 0, 0, 0 };
  static cilist io___37 = { 0, 9, 0, 0, 0 };
  static cilist io___38 = { 0, 9, 0, 0, 0 };
  static cilist io___40 = { 0, 6, 0, fmt_2050, 0 };
  static cilist io___47 = { 0, 6, 0, 0, 0 };

/* ---------------------------------------------------------------- */
/*     Reads Orr-Sommerfeld data file in binary or ascii format. */
/*     Data is spatial amplification complex wavenumber */
/*           ar(Re,w,H)  ai(Re,w,H) */
/*     stored on a R,W,H grid */
/*           R = ln(Re) */
/*           W = ln(w) - 0.5 ln(Re) */
/*           H = H */

/*     Input */
/*       FNAME   name of data file to be read */
/*       IFORM   -1=unknown */
/*                0=binary */
/*                1=ascii */

/*     Output */
/*       N       number of points across BL,  i=1..N */
/*       NMAX      max dimension of N */
/*       ETA(i)  BL y coordinate */
/*       U(i)    velocity profile */
/*       S(i)    shear profile dU/deta */
/*       NRP    number of RTL values, r=1..NRP */
/*       NWP    number of WSL values, w=1..NWP */
/*       RTL(r)  R  values */
/*       WSL(w)  W  values */
/*       HH      H  value */
/*       AR(r,w) real wavenumber */
/*       AI(r,w) imaginary wavenumber */

/* ---------------------------------------------------------------- */
  /* Parameter adjustments */
  --s;
  --u;
  --eta;
  --rtl;
  ai_dim1 = *nrx;
  ai_offset = 1 + ai_dim1;
  ai -= ai_offset;
  ar_dim1 = *nrx;
  ar_offset = 1 + ar_dim1;
  ar -= ar_offset;
  --wsl;

  if (*iform <= -1) 
  {
    /* ----- first assume it's an ascii file */
    kform = 1;

    /* ----- try reading it as a binary */
    o__1.oerr = 1;
    o__1.ounit = 9;
    o__1.ofnmlen = fname_len;
    o__1.ofnm = fname;
    o__1.orl = 0;
    o__1.osta = "OLD";
    o__1.oacc = 0;
    o__1.ofm = "UNFORMATTED";
    o__1.oblnk = 0;
    i__1 = f_open(&o__1);
    if (i__1 != 0) {
      goto L1001;
    }
  
    i__1 = s_rsue(&io___10);
    if (i__1 != 0) {
      goto L11;
    }
    i__1 = do_uio(&c__1, (char *)&ntest, (ftnlen)sizeof(integer));
    if (i__1 != 0) {
      goto L11;
    }
    i__1 = do_uio(&c__1, (char *)&htest, (ftnlen)sizeof(real));
    if (i__1 != 0) {
      goto L11;
    }
    i__1 = e_rsue();
    if (i__1 != 0) {
      goto L11;
    }

    if (ntest >= 1 && ntest <= *nmax) {
      /* ------ point index within bounds... looks like it's binary */
      kform = 0;
    }

    /*<  11    CLOSE(9) >*/
L11:
    cl__1.cerr = 0;
    cl__1.cunit = 9;
    cl__1.csta = 0;
    f_clos(&cl__1);
  } 
  else 
  {
    kform = *iform;
  }


  /*<       K = INDEX(FNAME,' ') - 1 >*/
  k = i_indx(fname, " ", fname_len, (ftnlen)1) - 1;

  /*<       IF(KFORM.EQ.0) THEN >*/
  if (kform == 0) {
    /* ----- binary format */
    /*<        FNAME = FNAME(1:K) >*/
    s_copy(fname, fname, fname_len, k);
    /*<        WRITE(*,*) FNAME, '   binary' >*/
    s_wsle(&io___14);
    do_lio(&c__9, &c__1, fname, fname_len);
    do_lio(&c__9, &c__1, "   binary", (ftnlen)9);
    e_wsle();

    /*<        OPEN(9,FILE=FNAME,STATUS='OLD',FORM='UNFORMATTED',ERR=1001) >*/
    o__1.oerr = 1;
    o__1.ounit = 9;
    o__1.ofnmlen = fname_len;
    o__1.ofnm = fname;
    o__1.orl = 0;
    o__1.osta = "OLD";
    o__1.oacc = 0;
    o__1.ofm = "UNFORMATTED";
    o__1.oblnk = 0;
    i__1 = f_open(&o__1);
    if (i__1 != 0) {
      goto L1001;
    }
    /*<        READ(9,ERR=1001) N, HH >*/
    i__1 = s_rsue(&io___15);
    if (i__1 != 0) {
      goto L1001;
    }
    i__1 = do_uio(&c__1, (char *)&(*n), (ftnlen)sizeof(integer));
    if (i__1 != 0) {
      goto L1001;
    }
    i__1 = do_uio(&c__1, (char *)&(*hh), (ftnlen)sizeof(real));
    if (i__1 != 0) {
      goto L1001;
    }
    i__1 = e_rsue();
    if (i__1 != 0) {
      goto L1001;
    }
    /*<        READ(9) (ETA(I),I=1, N) >*/
    s_rsue(&io___16);
    i__1 = *n;
    for (i__ = 1; i__ <= i__1; ++i__) {
      do_uio(&c__1, (char *)&eta[i__], (ftnlen)sizeof(real));
    }
    e_rsue();
    /*<        READ(9) (U(I)  ,I=1, N) >*/
    s_rsue(&io___18);
    i__1 = *n;
    for (i__ = 1; i__ <= i__1; ++i__) {
      do_uio(&c__1, (char *)&u[i__], (ftnlen)sizeof(real));
    }
    e_rsue();
    /*<        READ(9) (S(I)  ,I=1, N) >*/
    s_rsue(&io___19);
    i__1 = *n;
    for (i__ = 1; i__ <= i__1; ++i__) {
      do_uio(&c__1, (char *)&s[i__], (ftnlen)sizeof(real));
    }
    e_rsue();
    /*<        READ(9) NRP, NWP >*/
    s_rsue(&io___20);
    do_uio(&c__1, (char *)&(*nrp), (ftnlen)sizeof(integer));
    do_uio(&c__1, (char *)&(*nwp), (ftnlen)sizeof(integer));
    e_rsue();
    /*<        READ(9) (RTL(IR),IR=1,NRP) >*/
    s_rsue(&io___21);
    i__1 = *nrp;
    for (ir = 1; ir <= i__1; ++ir) {
      do_uio(&c__1, (char *)&rtl[ir], (ftnlen)sizeof(real));
    }
    e_rsue();
    /*<        READ(9) (WSL(IW),IW=1,NWP) >*/
    s_rsue(&io___23);
    i__1 = *nwp;
    for (iw = 1; iw <= i__1; ++iw) {
      do_uio(&c__1, (char *)&wsl[iw], (ftnlen)sizeof(real));
    }
    e_rsue();

    /*<        DO IW=1, NWP >*/
    i__1 = *nwp;
    for (iw = 1; iw <= i__1; ++iw) 
    {
      /*<          READ(9,END=15) (AR(IR,IW),IR=1,NRP) >*/
      i__2 = s_rsue(&io___25);
      if (i__2 != 0) 
      {
        goto L15;
      }
      i__3 = *nrp;
      for (ir = 1; ir <= i__3; ++ir) 
      {
        i__2 = do_uio(&c__1, (char *)&ar[ir + iw * ar_dim1], (ftnlen)
            sizeof(real));
        if (i__2 != 0) 
        {
          goto L15;
        }
      }
      i__2 = e_rsue();
      if (i__2 != 0) 
      {
        goto L15;
      }
      /*<          READ(9,END=15) (AI(IR,IW),IR=1,NRP) >*/
      i__2 = s_rsue(&io___26);
      if (i__2 != 0) 
      {
        goto L15;
      }
      i__3 = *nrp;
      for (ir = 1; ir <= i__3; ++ir) 
      {
        i__2 = do_uio(&c__1, (char *)&ai[ir + iw * ai_dim1], (ftnlen)
            sizeof(real));
        if (i__2 != 0) 
        {
          goto L15;
        }
      }
      i__2 = e_rsue();
      if (i__2 != 0) 
      {
        goto L15;
      }
    }
    goto L30;

    /*<  15    CONTINUE >*/
L15:
    iwlast = iw - 1;
    s_wsle(&io___28);
    do_lio(&c__9, &c__1, "Map incomplete.  Last complete frequency index:"
          , (ftnlen)47);
    do_lio(&c__3, &c__1, (char *)&iwlast, (ftnlen)sizeof(integer));
    e_wsle();
  } 
  else 
  {
    /* ----- ascii format */
    s_copy(fname, fname, fname_len, k);
    /*<        WRITE(*,*) FNAME, '   ascii' >*/
    s_wsle(&io___29);
    do_lio(&c__9, &c__1, fname, fname_len);
    do_lio(&c__9, &c__1, "   ascii", (ftnlen)8);
    e_wsle();

    /*<        OPEN(9,FILE=FNAME,STATUS='OLD') >*/
    o__1.oerr = 0;
    o__1.ounit = 9;
    o__1.ofnmlen = fname_len;
    o__1.ofnm = fname;
    o__1.orl = 0;
    o__1.osta = "OLD";
    o__1.oacc = 0;
    o__1.ofm = 0;
    o__1.oblnk = 0;
    f_open(&o__1);
    /*<        READ(9,*) N, HH >*/
    s_rsle(&io___30);
    do_lio(&c__3, &c__1, (char *)&(*n), (ftnlen)sizeof(integer));
    do_lio(&c__4, &c__1, (char *)&(*hh), (ftnlen)sizeof(real));
    e_rsle();
    /*<        READ(9,*) (ETA(I),I=1, N) >*/
    s_rsle(&io___31);
    i__1 = *n;
    for (i__ = 1; i__ <= i__1; ++i__) 
    {
      do_lio(&c__4, &c__1, (char *)&eta[i__], (ftnlen)sizeof(real));
    }
    e_rsle();
    /*<        READ(9,*) (U(I)  ,I=1, N) >*/
    s_rsle(&io___32);
    i__1 = *n;
    for (i__ = 1; i__ <= i__1; ++i__) 
    {
      do_lio(&c__4, &c__1, (char *)&u[i__], (ftnlen)sizeof(real));
    }
    e_rsle();
    /*<        READ(9,*) (S(I)  ,I=1, N) >*/
    s_rsle(&io___33);
    i__1 = *n;
    for (i__ = 1; i__ <= i__1; ++i__) 
    {
      do_lio(&c__4, &c__1, (char *)&s[i__], (ftnlen)sizeof(real));
    }
    e_rsle();
    /*<        READ(9,*) NRP, NWP >*/
    s_rsle(&io___34);
    do_lio(&c__3, &c__1, (char *)&(*nrp), (ftnlen)sizeof(integer));
    do_lio(&c__3, &c__1, (char *)&(*nwp), (ftnlen)sizeof(integer));
    e_rsle();
    /*<        READ(9,*) (RTL(IR),IR=1,NRP) >*/
    s_rsle(&io___35);
    i__1 = *nrp;
    for (ir = 1; ir <= i__1; ++ir) 
    {
      do_lio(&c__4, &c__1, (char *)&rtl[ir], (ftnlen)sizeof(real));
    }
    e_rsle();
    /*<        READ(9,*) (WSL(IW),IW=1,NWP) >*/
    s_rsle(&io___36);
    i__1 = *nwp;
    for (iw = 1; iw <= i__1; ++iw) 
    {
      do_lio(&c__4, &c__1, (char *)&wsl[iw], (ftnlen)sizeof(real));
    }
    e_rsle();

    /*<        DO IW=1, NWP >*/
    i__1 = *nwp;
    for (iw = 1; iw <= i__1; ++iw) 
    {
      /*<          READ(9,*) (AR(IR,IW),IR=1,NRP) >*/
      s_rsle(&io___37);
      i__2 = *nrp;
      for (ir = 1; ir <= i__2; ++ir) 
      {
        do_lio(&c__4, &c__1, (char *)&ar[ir + iw * ar_dim1], (ftnlen)
              sizeof(real));
      }
      e_rsle();
      /*<          READ(9,*) (AI(IR,IW),IR=1,NRP) >*/
      s_rsle(&io___38);
      i__2 = *nrp;
      for (ir = 1; ir <= i__2; ++ir) 
      {
        do_lio(&c__4, &c__1, (char *)&ai[ir + iw * ai_dim1], (ftnlen)
              sizeof(real));
      }
      e_rsle();
    }
  }

L30:
    /*<       CLOSE(9) >*/
    cl__1.cerr = 0;
    cl__1.cunit = 9;
    cl__1.csta = 0;
    f_clos(&cl__1);
    /*<       GEO = (ETA(3)-ETA(2)) / (ETA(2)-ETA(1)) >*/
    geo = (eta[3] - eta[2]) / (eta[2] - eta[1]);
    s_wsfe(&io___40);
    do_fio(&c__1, (char *)&(*n), (ftnlen)sizeof(integer));
    do_fio(&c__1, (char *)&(*hh), (ftnlen)sizeof(real));
    do_fio(&c__1, (char *)&eta[*n], (ftnlen)sizeof(real));
    do_fio(&c__1, (char *)&geo, (ftnlen)sizeof(real));
    e_wsfe();
    
    *iform = 1;

    /* ---- re-order if needed to make RTL and WSL monotonically increasing */
    if (rtl[1] > rtl[*nrp]) 
    {
      i__1 = *nrp / 2;
      for (ir = 1; ir <= i__1; ++ir) 
      {
        irback = *nrp - ir + 1;

        rtemp = rtl[ir];
        rtl[ir] = rtl[irback];
        rtl[irback] = rtemp;

        /*<          DO IW=1, NWP >*/
        i__2 = *nwp;
        for (iw = 1; iw <= i__2; ++iw) 
        {
          artemp = ar[ir + iw * ar_dim1];
          aitemp = ai[ir + iw * ai_dim1];
          ar[ir + iw * ar_dim1] = ar[irback + iw * ar_dim1];
          ai[ir + iw * ai_dim1] = ai[irback + iw * ai_dim1];
          ar[irback + iw * ar_dim1] = artemp;
          ai[irback + iw * ai_dim1] = aitemp;
        }
      }
    }

    /*<       IF(WSL(1) .GT. WSL(NWP)) THEN >*/
    if (wsl[1] > wsl[*nwp]) 
    {
      /*<        DO IW=1, NWP/2 >*/
      i__1 = *nwp / 2;
      for (iw = 1; iw <= i__1; ++iw) 
      {
        iwback = *nwp - iw + 1;

        wtemp = wsl[iw];
        wsl[iw] = wsl[iwback];
        wsl[iwback] = wtemp;

        i__2 = *nrp;
        for (ir = 1; ir <= i__2; ++ir) 
        {
          /*<            ARTEMP = AR(IR,IW) >*/
          artemp = ar[ir + iw * ar_dim1];
          /*<            AITEMP = AI(IR,IW) >*/
          aitemp = ai[ir + iw * ai_dim1];
          /*<            AR(IR,IW) = AR(IR,IWBACK) >*/
          ar[ir + iw * ar_dim1] = ar[ir + iwback * ar_dim1];
          /*<            AI(IR,IW) = AI(IR,IWBACK) >*/
          ai[ir + iw * ai_dim1] = ai[ir + iwback * ai_dim1];
          /*<            AR(IR,IWBACK) = ARTEMP >*/
          ar[ir + iwback * ar_dim1] = artemp;
          /*<            AI(IR,IWBACK) = AITEMP >*/
          ai[ir + iwback * ai_dim1] = aitemp;
        }
      }
    }
    return 0;

    /*<  1001 CONTINUE >*/
L1001:
    s_wsle(&io___47);
    do_lio(&c__9, &c__1, "File open error.", (ftnlen)16);
    e_wsle();
    return 0;
} /* reados1_ */

int writos1_(
             char *fname, 
             int *iform, 
             int *n, 
             int *nmax, 
             real *eta, 
             real *u, 
             real *s, 
             int *nrp, 
             int * nwp, 
             real *rtl, 
             real *wsl, 
             real *hh, 
             real *ar, 
             real *ai, 
             int *nrx,
             int *nwx, 
             ftnlen fname_len)
{
    /* System generated locals */
    address a__1[2];
    int ar_dim1, ar_offset, ai_dim1, ai_offset, i__1[2], i__2, i__3;
    olist o__1;
    cllist cl__1;
    alist al__1;

    /* Local variables */
    static int i__, k, ir, iw;

    /* Fortran I/O blocks */
    static cilist io___49 = { 0, 6, 0, 0, 0 };
    static cilist io___50 = { 1, 9, 0, 0, 0 };
    static cilist io___51 = { 0, 9, 0, 0, 0 };
    static cilist io___53 = { 0, 9, 0, 0, 0 };
    static cilist io___54 = { 0, 9, 0, 0, 0 };
    static cilist io___55 = { 0, 9, 0, 0, 0 };
    static cilist io___56 = { 0, 9, 0, 0, 0 };
    static cilist io___58 = { 0, 9, 0, 0, 0 };
    static cilist io___60 = { 0, 9, 0, 0, 0 };
    static cilist io___61 = { 0, 9, 0, 0, 0 };
    static cilist io___62 = { 0, 6, 0, 0, 0 };
    static cilist io___63 = { 0, 9, 0, 0, 0 };
    static cilist io___64 = { 0, 9, 0, 0, 0 };
    static cilist io___65 = { 0, 9, 0, 0, 0 };
    static cilist io___66 = { 0, 9, 0, 0, 0 };
    static cilist io___67 = { 0, 9, 0, 0, 0 };
    static cilist io___68 = { 0, 9, 0, 0, 0 };
    static cilist io___69 = { 0, 9, 0, 0, 0 };
    static cilist io___70 = { 0, 9, 0, 0, 0 };
    static cilist io___71 = { 0, 9, 0, 0, 0 };
    static cilist io___72 = { 0, 6, 0, 0, 0 };


/* ---------------------------------------------------------------- */
/*     Writes Orr-Sommerfeld data file in binary or ascii format. */
/*     Data is spatial amplification complex wavenumber */
/*           ar(Re,w,H)  ai(Re,w,H) */
/*     stored on a R,W,H grid */
/*           R = ln(Re) */
/*           W = ln(w) - 0.5 ln(Re) */
/*           H = H */

/*     Input */
/*       FNAME    name of data file to be written */
/*       IFORM    0=binary, ascii otherwise */
/*       N        number of points across BL,  i=1..N */
/*       NMAX     max dimension of N */
/*       ETA(i)   BL y coordinate */
/*       U(i)     velocity profile */
/*       S(i)     shear profile dU/deta */
/*       NRP      number of RTL values, r=1..NRP */
/*       NWP      number of WSL values, w=1..NWP */
/*       RTL(r)   R  values */
/*       WSL(w)   W  values */
/*       HH       H  value */
/*       AR(r,w)  real wavenumber */
/*       AI(r,w)  imaginary wavenumber */

/*     Output */
/*       written file */

/* ---------------------------------------------------------------- */
    
    /* Parameter adjustments */
    --s;
    --u;
    --eta;
    --rtl;
    ai_dim1 = *nrx;
    ai_offset = 1 + ai_dim1;
    ai -= ai_offset;
    ar_dim1 = *nrx;
    ar_offset = 1 + ar_dim1;
    ar -= ar_offset;
    --wsl;

    /* Function Body */
    k = i_indx(fname, " ", fname_len, (ftnlen)1) - 1;

    if (*iform == 0) 
    {
      /*<        FNAME = FNAME(1:K) // '.bin' >*/
      /* Writing concatenation */
      i__1[0] = k, a__1[0] = fname;
      i__1[1] = 4, a__1[1] = ".bin";
      s_cat(fname, a__1, i__1, &c__2, fname_len);
      /*<        WRITE(*,*) FNAME >*/
      s_wsle(&io___49);
      do_lio(&c__9, &c__1, fname, fname_len);
      e_wsle();

      /*<        OPEN(9,FILE=FNAME,STATUS='UNKNOWN',FORM='UNFORMATTED',ERR=1001) >*/
      o__1.oerr = 1;
      o__1.ounit = 9;
      o__1.ofnmlen = fname_len;
      o__1.ofnm = fname;
      o__1.orl = 0;
      o__1.osta = "UNKNOWN";
      o__1.oacc = 0;
      o__1.ofm = "UNFORMATTED";
      o__1.oblnk = 0;
      i__2 = f_open(&o__1);
      if (i__2 != 0) {
        goto L1001;
      }
      /*<        REWIND(9) >*/
      al__1.aerr = 0;
      al__1.aunit = 9;
      f_rew(&al__1);
      /*<        WRITE(9,ERR=1001) N, HH >*/
      i__2 = s_wsue(&io___50);
      if (i__2 != 0) {
        goto L1001;
      }
      i__2 = do_uio(&c__1, (char *)&(*n), (ftnlen)sizeof(integer));
      if (i__2 != 0) {
        goto L1001;
      }
      i__2 = do_uio(&c__1, (char *)&(*hh), (ftnlen)sizeof(real));
      if (i__2 != 0) {
        goto L1001;
      }
      i__2 = e_wsue();
      if (i__2 != 0) {
        goto L1001;
      }
      /*<        WRITE(9) (ETA(I),I=1, N) >*/
      s_wsue(&io___51);
      i__2 = *n;
      for (i__ = 1; i__ <= i__2; ++i__) {
        do_uio(&c__1, (char *)&eta[i__], (ftnlen)sizeof(real));
      }
      e_wsue();
      /*<        WRITE(9) (U(I)  ,I=1, N) >*/
      s_wsue(&io___53);
      i__2 = *n;
      for (i__ = 1; i__ <= i__2; ++i__) {
        do_uio(&c__1, (char *)&u[i__], (ftnlen)sizeof(real));
      }
      e_wsue();
      /*<        WRITE(9) (S(I)  ,I=1, N) >*/
      s_wsue(&io___54);
      i__2 = *n;
      for (i__ = 1; i__ <= i__2; ++i__) {
        do_uio(&c__1, (char *)&s[i__], (ftnlen)sizeof(real));
      }
      e_wsue();
      /*<        WRITE(9) NRP, NWP >*/
      s_wsue(&io___55);
      do_uio(&c__1, (char *)&(*nrp), (ftnlen)sizeof(integer));
      do_uio(&c__1, (char *)&(*nwp), (ftnlen)sizeof(integer));
      e_wsue();
      /*<        WRITE(9) (RTL(IR),IR=1,NRP) >*/
      s_wsue(&io___56);
      i__2 = *nrp;
      for (ir = 1; ir <= i__2; ++ir) {
        do_uio(&c__1, (char *)&rtl[ir], (ftnlen)sizeof(real));
      }
      e_wsue();
      /*<        WRITE(9) (WSL(IW),IW=1,NWP) >*/
      s_wsue(&io___58);
      i__2 = *nwp;
      for (iw = 1; iw <= i__2; ++iw) {
        do_uio(&c__1, (char *)&wsl[iw], (ftnlen)sizeof(real));
      }
      e_wsue();

      /*<        DO IW=1, NWP >*/
      i__2 = *nwp;
      for (iw = 1; iw <= i__2; ++iw) {
        /*<          WRITE(9) (AR(IR,IW),IR=1,NRP) >*/
        s_wsue(&io___60);
        i__3 = *nrp;
        for (ir = 1; ir <= i__3; ++ir) {
          do_uio(&c__1, (char *)&ar[ir + iw * ar_dim1], (ftnlen)sizeof(
                  real));
        }
        e_wsue();
        /*<          WRITE(9) (AI(IR,IW),IR=1,NRP) >*/
        s_wsue(&io___61);
        i__3 = *nrp;
        for (ir = 1; ir <= i__3; ++ir) {
          do_uio(&c__1, (char *)&ai[ir + iw * ai_dim1], (ftnlen)sizeof(
                  real));
        }
        e_wsue();
        /*<        ENDDO >*/
      }

      /*<       ELSE >*/
    } 
    else 
    {
      /*<        FNAME = FNAME(1:K) >*/
      s_copy(fname, fname, fname_len, k);
      /*<        WRITE(*,*) FNAME >*/
      s_wsle(&io___62);
      do_lio(&c__9, &c__1, fname, fname_len);
      e_wsle();

      /*<        OPEN(9,FILE=FNAME,STATUS='UNKNOWN') >*/
      o__1.oerr = 0;
      o__1.ounit = 9;
      o__1.ofnmlen = fname_len;
      o__1.ofnm = fname;
      o__1.orl = 0;
      o__1.osta = "UNKNOWN";
      o__1.oacc = 0;
      o__1.ofm = 0;
      o__1.oblnk = 0;
      f_open(&o__1);
      /*<        REWIND(9) >*/
      al__1.aerr = 0;
      al__1.aunit = 9;
      f_rew(&al__1);
      /*<        WRITE(9,*) N, HH >*/
      s_wsle(&io___63);
      do_lio(&c__3, &c__1, (char *)&(*n), (ftnlen)sizeof(integer));
      do_lio(&c__4, &c__1, (char *)&(*hh), (ftnlen)sizeof(real));
      e_wsle();
      /*<        WRITE(9,*) (ETA(I),I=1, N) >*/
      s_wsle(&io___64);
      i__2 = *n;
      for (i__ = 1; i__ <= i__2; ++i__) {
        do_lio(&c__4, &c__1, (char *)&eta[i__], (ftnlen)sizeof(real));
      }
      e_wsle();
      /*<        WRITE(9,*) (U(I)  ,I=1, N) >*/
      s_wsle(&io___65);
      i__2 = *n;
      for (i__ = 1; i__ <= i__2; ++i__) {
        do_lio(&c__4, &c__1, (char *)&u[i__], (ftnlen)sizeof(real));
      }
      e_wsle();
      /*<        WRITE(9,*) (S(I)  ,I=1, N) >*/
      s_wsle(&io___66);
      i__2 = *n;
      for (i__ = 1; i__ <= i__2; ++i__) {
        do_lio(&c__4, &c__1, (char *)&s[i__], (ftnlen)sizeof(real));
      }
      e_wsle();
      /*<        WRITE(9,*) NRP, NWP >*/
      s_wsle(&io___67);
      do_lio(&c__3, &c__1, (char *)&(*nrp), (ftnlen)sizeof(integer));
      do_lio(&c__3, &c__1, (char *)&(*nwp), (ftnlen)sizeof(integer));
      e_wsle();
      /*<        WRITE(9,*) (RTL(IR),IR=1,NRP) >*/
      s_wsle(&io___68);
      i__2 = *nrp;
      for (ir = 1; ir <= i__2; ++ir) {
        do_lio(&c__4, &c__1, (char *)&rtl[ir], (ftnlen)sizeof(real));
      }
      e_wsle();
      /*<        WRITE(9,*) (WSL(IW),IW=1,NWP) >*/
      s_wsle(&io___69);
      i__2 = *nwp;
      for (iw = 1; iw <= i__2; ++iw) {
        do_lio(&c__4, &c__1, (char *)&wsl[iw], (ftnlen)sizeof(real));
      }
      e_wsle();

      /*<        DO IW=1, NWP >*/
      i__2 = *nwp;
      for (iw = 1; iw <= i__2; ++iw) 
      {
        /*<          WRITE(9,*) (AR(IR,IW),IR=1,NRP) >*/
        s_wsle(&io___70);
        i__3 = *nrp;
        for (ir = 1; ir <= i__3; ++ir) {
          do_lio(&c__4, &c__1, (char *)&ar[ir + iw * ar_dim1], (ftnlen)
                sizeof(real));
        }
        e_wsle();
        /*<          WRITE(9,*) (AI(IR,IW),IR=1,NRP) >*/
        s_wsle(&io___71);
        i__3 = *nrp;
        for (ir = 1; ir <= i__3; ++ir) {
          do_lio(&c__4, &c__1, (char *)&ai[ir + iw * ai_dim1], (ftnlen)
                sizeof(real));
        }
        e_wsle();
      }
    }

    /*<       CLOSE(9) >*/
    cl__1.cerr = 0;
    cl__1.cunit = 9;
    cl__1.csta = 0;
    f_clos(&cl__1);
    return 0;

    /*<  1001 CONTINUE >*/
L1001:
    /*<       WRITE(*,*) 'File open error.' >*/
    s_wsle(&io___72);
    do_lio(&c__9, &c__1, "File open error.", (ftnlen)16);
    e_wsle();
    return 0;
} /* writos1_ */

